@@ -1,15 +1,15 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 __version__ = '$Id$'
 from tools.docco.rl_doc_utils import *
 from reportlab.platypus.tableofcontents import TableOfContents
-from datetime import datetime
 import reportlab
+from reportlab.lib.utils import TimeStamp
 
 title("ReportLab PDF Library")
 title("User Guide")
 centred('ReportLab Version ' + reportlab.Version)
-centred(datetime.now().strftime('Document generated on %Y/%m/%d %H:%M:%S %Z'))
+centred(TimeStamp().datetime.strftime('Document generated on %Y/%m/%d %H:%M:%S %Z'))
 
 nextTemplate("TOC")
 
@@ -48,7 +48,7 @@ where to go for orientation.
 disc("""
 This manual does not cover 100% of the features, but should explain all
 the main concepts and help you get started, and point you at other
-learning resources. 
+learning resources.
 After working your way through this, you should be ready to begin
 writing programs to produce sophisticated reports.
 """)
@@ -180,7 +180,7 @@ Python is copyrighted but <b>freely usable and distributable, even for commercia
 heading2("Acknowledgements")
 disc("""Many people have contributed to ReportLab.  We would like to thank in particular 
 (in alphabetical order): 
-Albertas Agejevas, 
+<nobr>Albertas Agejevas, 
 Alex Buck, 
 Andre Reitz, 
 Andrew Cutler,
@@ -189,11 +189,14 @@ Ben Echols,
 Benjamin Dumke,
 Benn B,
 Chad Miller, 
+Chris Buergi,
 Chris Lee, 
 Christian Jacobs, 
 Dinu Gherman,
+Edward Greve,
 Eric Johnson,
 Felix Labrecque,  
+Fubu @ bitbucket,
 Gary Poster, 
 Germán M. Bravo,
 Guillaume Francois, 
@@ -207,14 +210,17 @@ Jerome Alet,
 Jerry Casiano,
 Jorge Godoy,
 Keven D Smith,
+Kyle MacFarlane,
 Magnus Lie Hetland,
 Marcel Tromp, 
 Marius Gedminas,
+Mark de Wit,
 Matthew Duggan,
 Matthias Kirst,
 Matthias Klose,
 Max M, 
 Michael Egorov,
+Michael Spector,
 Mike Folwell,
 Mirko Dziadzka,
 Moshe Wagner,
@@ -228,22 +234,24 @@ Robert Alsina,
 Robert Hölzl,
 Robert Kern,
 Ron Peleg,
+Ruby Yocum,
 Simon King,
 Stephan Richter,
 Steve Halasz, 
+Stoneleaf @ bitbucket,
 T Blatter,
 Tim Roberts,
 Tomasz Swiderski,
 Ty Sarna,
 Volker Haas,
-Yoann Roman, 
+Yoann Roman,</nobr>
 and many more.""")
 
 
 disc("""Special thanks go to Just van Rossum for his valuable assistance with
 font technicalities.""")
 
-disc("""Moshe Wagner and Hosam Aly deserve a huge thanks for contributing to the RTL patch, which is not yet on thr trunk.""")
+disc("""Moshe Wagner and Hosam Aly deserve a huge thanks for contributing to the RTL patch, which is not yet on the trunk.""")
 
 disc("""Marius Gedminas deserves a big hand for contributing the work on TrueType fonts and we
 are glad to include these in the toolkit. Finally we thank Michal Kosmulski for the DarkGarden font
