@@ -1,6 +1,6 @@
-#Copyright ReportLab Europe Ltd. 2000-2016
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/docs/userguide/ch6_tables.py
+#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/docs/userguide/ch6_tables.py
 from tools.docco.rl_doc_utils import *
 from reportlab.platypus import Image,ListFlowable, ListItem
 import reportlab
@@ -57,7 +57,7 @@ heading2('$Table$ User Methods')
 disc("""These are the main methods which are of interest to the client programmer.""")
 
 heading4("""$Table(data, colWidths=None, rowHeights=None, style=None, splitByRow=1,
-repeatRows=0, repeatCols=0)$""")
+repeatRows=0, repeatCols=0, rowSplitRange=None, spaceBefore=None, spaceAfter=None)$""")
 
 disc("""The $data$ argument is a sequence of sequences of cell values each of which
 should be convertible to a string value using the $str$ function or should be a Flowable instance (such as a $Paragraph$) or a list (or tuple) of such instances.
@@ -91,6 +91,8 @@ disc("""The $repeatRows$ argument specifies the number or a tuple of leading row
 that should be repeated when the $Table$ is asked to split itself. If it is a tuple it should specify which of the leading rows should be repeated; this allows
 for cases where the first appearance of the table hsa more leading rows than later split parts.
 The $repeatCols$ argument is currently ignored as a $Table$ cannot be split by column.""")
+disc("""The $spaceBefore$ &amp; $spaceAfter$ arguments may be used to put extra space before or after the table when renedered in a $platypus$ story.""")
+disc("""The $rowSplitRange$ argument may be used to control the splitting of the table to a subset of its rows; that can be to prevent splitting too close to the beginning or end of the table.""")
 heading4('$Table.setStyle(tblStyle)$')
 disc("""
 This method applies a particular instance of class $TableStyle$ (discussed below)
@@ -151,7 +153,7 @@ heading3("""$TableStyle$ Cell Formatting Commands""")
 disc("""The cell formatting commands all begin with an identifier, followed by
 the start and stop cell definitions and the perhaps other arguments.
 the cell formatting commands are:""")
-eg("""
+npeg("""
 FONT                    - takes fontname, optional fontsize and optional leading.
 FONTNAME (or FACE)      - takes fontname.
 FONTSIZE (or SIZE)      - takes fontsize in points; leading may get out of sync.
@@ -162,7 +164,10 @@ LEFTPADDING             - takes an integer, defaults to 6.
 RIGHTPADDING            - takes an integer, defaults to 6.
 BOTTOMPADDING           - takes an integer, defaults to 3.
 TOPPADDING              - takes an integer, defaults to 3.
-BACKGROUND              - takes a color.
+BACKGROUND              - takes a color defined by an object, string name or numeric tuple/list,
+                          or takes a list/tuple describing a desired gradient fill which should
+                          contain three elements of the form [DIRECTION, startColor, endColor]
+                          where DIRECTION is either VERTICAL or HORIZONTAL.
 ROWBACKGROUNDS          - takes a list of colors to be used cyclically.
 COLBACKGROUNDS          - takes a list of colors to be used cyclically.
 VALIGN                  - takes one of TOP, MIDDLE or the default BOTTOM
@@ -307,7 +312,7 @@ eg("""
 SPAN, (sc,sr), (ec,er)
 """)
 disc("""indicates that the cells in columns $sc$ - $ec$ and rows $sr$ - $er$ should be combined into a super cell
-with contents determined by the cell $(sc, sr)$. The other cells should be present, but should contain empty strings 
+with contents determined by the cell $(sc, sr)$. The other cells should be present, but should contain empty strings
 or you may get unexpected results.
 """)
 EmbeddedCode("""
@@ -326,10 +331,19 @@ t=Table(data,style=[
 
 disc("""notice that we don't need to be conservative with our $GRID$ command. The spanned cells are not drawn through.
 """)
+heading3("""$TableStyle$ Miscellaneous Commands""")
+disc("""To control $Table$ splitting the $NOSPLIT$ command may be used
+The style specification
+""")
+eg("""
+NOSPLIT, (sc,sr), (ec,er)
+""")
+disc("""demands that the cells in columns $sc$ - $ec$ and rows $sr$ - $er$ may not be split.""")
+
 heading3("""Special $TableStyle$ Indeces""")
 disc("""In any style command the first row index may be set to one of the special strings
 $'splitlast'$ or $'splitfirst'$ to indicate that the style should be used only for the last row of
-a split table, or the first row of a continuation. This allows splitting tables with nicer effects around the split.""")  
+a split table, or the first row of a continuation. This allows splitting tables with nicer effects around the split.""")
 
 heading1("""Programming $Flowables$""")
 
@@ -342,7 +356,7 @@ disc("""Assigns a variable of name $var$ to the expression $expr$. E.g.:""")
 eg("""
 DocAssign('i',3)
 """)
-                
+
 heading2("""$DocExec(self, stmt, lifetime='forever')$""")
 
 disc("""Executes the statement $stmt$. E.g.:""")
@@ -414,7 +428,7 @@ disc("""
 The line will be split on any single character defined in $splitChars$. If no value is provided for this property, the line will be split on any of the following standard characters: space, colon, full stop, semi-colon, coma, hyphen, forward slash, back slash, left parenthesis, left square bracket and left curly brace
 """)
 disc("""
-Characters can be automatically inserted at the beginning of each line that has been created. You can set the property $newLineChars$ to the characters you want to use. 
+Characters can be automatically inserted at the beginning of each line that has been created. You can set the property $newLineChars$ to the characters you want to use.
 """)
 EmbeddedCode("""
 from reportlab.lib.styles import getSampleStyleSheet
@@ -458,7 +472,8 @@ t=XPreformatted(text,normalStyle,dedent=3)
 """)
 
 heading2("""$Image(filename, width=None, height=None)$""")
-disc("""Create a flowable which will contain the image defined by the data in file $filename$.
+disc("""Create a flowable which will contain the image defined by the data in file $filename$ which can be
+filepath, file like object or an instance of a $reportlab.graphics.shapes.Drawing$.
 The default <b>PDF</b> image type <i>jpeg</i> is supported and if the <b>PIL</b> extension to <b>Python</b>
 is installed the other image types can also be handled. If $width$ and or $height$ are specified
 then they determine the dimension of the displayed image in <i>points</i>. If either dimension is
@@ -723,3 +738,38 @@ Paragraph("Item no.4", style),
 bulletType='i'
 )
              """)
+
+disc("""To cope with nesting the $start$ parameter can be set to a list of possible starts; for $ul$ acceptable starts are any unicode character or specific names known to flowables.py eg
+$bulletchar$, $circle$, $square$, $disc$, $diamond$, $diamondwx$, $rarrowhead$, $sparkle$, $squarelrs$ or  $blackstar$. For $ol$ the $start$ can be any character from $'1iaAI'$ to indicate different number styles.
+""")
+
+heading2("""$BalancedColumns()$""")
+disc("""Use the $BalancedColumns$ class to make a flowable that splits its content flowables into two or more roughly equal sized columns.
+Effectively $n$ frames are synthesized to take the content and the flowable tries to balance the content between them. The created frames
+will be split when the total height is too large and the split will maintain the balance. 
+""")
+eg("""
+from reportlab.platypus.flowables import BalancedColumns
+from reportlab.platypus.frames import ShowBoundaryValue
+F = [
+    list of flowables........
+    ]
+story.append(
+    Balanced(
+        F,          #the flowables we are balancing
+        nCols = 2,  #the number of columns
+        needed = 72,#the minimum space needed by the flowable
+        spacBefore = 0,
+        spaceAfter = 0,
+        showBoundary = None,    #optional boundary showing
+        leftPadding=None,       #these override the created frame
+        rightPadding=None,      #paddings if specified else the
+        topPadding=None,        #default frame paddings
+        bottomPadding=None,     #are used
+        innerPadding=None,      #the gap between frames if specified else
+                                #use max(leftPadding,rightPadding)
+        name='',                #for identification purposes when stuff goes awry
+        endSlack=0.1,           #height disparity allowance ie 10% of available height
+        )
+    )   
+""")
