@@ -1,6 +1,6 @@
 #Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/shapes.py
+#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/graphics/shapes.py
 
 __version__='3.3.0'
 __doc__='''Core of the graphics library - defines Drawing and Shapes'''
@@ -21,6 +21,7 @@ from reportlab.lib.rl_accel import fp_str
 from reportlab.pdfbase.pdfmetrics import stringWidth
 from reportlab.lib.fonts import tt2ps
 from reportlab.pdfgen.canvas import FILL_EVEN_ODD, FILL_NON_ZERO
+from reportlab import xrange
 _baseGFontNameB = tt2ps(_baseGFontName,1,0)
 _baseGFontNameI = tt2ps(_baseGFontName,0,1)
 _baseGFontNameBI = tt2ps(_baseGFontName,1,1)
@@ -1330,7 +1331,7 @@ class Wedge(SolidShape):
                 a(centerx+radius1*c)
                 a(centery+yradius1*s)
         if self.annular:
-            P = Path(fillMode=self.fillMode)
+            P = Path(fillMode=getattr(self,'fillMode',None))
             P.moveTo(points[0],points[1])
             for x in xrange(2,2*n,2):
                 P.lineTo(points[x],points[x+1])
@@ -1481,6 +1482,11 @@ class UserNode(_DrawTimeResizeable):
 
         raise NotImplementedError("this method must be redefined by the user/programmer")
 
+class DirectDraw(Shape):
+    """try to draw directly on the canvas"""
+    def drawDirectly(self,canvas):
+        raise NotImplementedError("this method must be redefined by the user/programmer")
+
 def test():
     r = Rect(10,10,200,50)
     import pprint
