@@ -1,7 +1,7 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/lib/colors.py
-__version__=''' $Id$ '''
+#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/lib/colors.py
+__version__='3.3.0'
 __doc__='''Defines standard colour-handling classes and colour names.
 
 We define standard classes to hold colours in two models:  RGB and CMYK.
@@ -40,7 +40,7 @@ Traceback (most recent call last):
 ValueError: css color 'pcmyka(100,0,0,0)' has wrong number of components
 '''
 import math, re, functools
-from reportlab import isPy3
+from reportlab import isPy3, cmp
 from reportlab.lib.rl_accel import fp_str
 from reportlab.lib.utils import asNative, isStr
 import collections
@@ -148,6 +148,10 @@ class Color:
         return self.alpha
 if isPy3: Color = functools.total_ordering(Color)
 
+def opaqueColor(c):
+    '''utility to check we have a color that's not fully transparent'''
+    return isinstance(c,Color) and c.alpha>0
+
 class CMYKColor(Color):
     """This represents colors using the CMYK (cyan, magenta, yellow, black)
     model commonly used in professional printing.  This is implemented
@@ -363,7 +367,8 @@ def HexColor(val, htmlOnly=False, hasAlpha=False):
 
     """ #" for emacs
 
-    if isinstance(val,str):
+    if isStr(val):
+        val = asNative(val)
         b = 10
         if val[:1] == '#':
             val = val[1:]
