@@ -1,7 +1,7 @@
 #Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/pdfbase/pdfdoc.py
-__version__='3.3.0'
+#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/pdfbase/pdfdoc.py
+__version__='3.4.1'
 __doc__="""
 The module pdfdoc.py handles the 'outer structure' of PDF documents, ensuring that
 all objects are properly cross-referenced and indexed to the nearest byte.  The
@@ -17,8 +17,8 @@ classes are made available elsewhere for users to manipulate.
 import types, binascii, codecs, time
 from collections import OrderedDict
 from reportlab.pdfbase import pdfutils
-from reportlab import rl_config
-from reportlab.lib.utils import import_zlib, open_for_read, makeFileName, isSeq, isBytes, isUnicode, _digester, isStr, bytestr, isPy3, annotateException
+from reportlab import rl_config, ascii
+from reportlab.lib.utils import import_zlib, open_for_read, makeFileName, isSeq, isBytes, isUnicode, _digester, isStr, bytestr, isPy3, annotateException, TimeStamp
 from reportlab.lib.rl_accel import escapePDF, fp_str, asciiBase85Encode, asciiBase85Decode
 from reportlab.pdfbase import pdfmetrics
 from hashlib import md5
@@ -148,10 +148,8 @@ class PDFDocument(PDFObject):
         # signature for creating PDF ID
         sig = self.signature = md5()
         sig.update(b"a reportlab document")
-        if not self.invariant:
-            cat = _getTimeStamp()
-        else:
-            cat = 946684800.0
+        self._timeStamp = TimeStamp(self.invariant)
+        cat = self._timeStamp.t
         cat = ascii(cat)
         sig.update(bytestr(cat)) # initialize with timestamp digest
         # mapping of internal identifier ("Page001") to PDF objectnumber and generation number (34, 0)
@@ -165,14 +163,8 @@ class PDFDocument(PDFObject):
         cat = self.Catalog = self._catalog = PDFCatalog()
         pages = self.Pages = PDFPages()
         cat.Pages = pages
-        if dummyoutline:
-            outlines = PDFOutlines0()
-        else:
-            outlines = PDFOutlines()
-        self.Outlines = self.outline = outlines
-        cat.Outlines = outlines
+        self.outline = self.Outlines = cat.Outlines = PDFOutlines0() if dummyoutline else PDFOutlines()
         self.info = PDFInfo()
-        self.info.invariant = self.invariant
         #self.Reference(self.Catalog)
         #self.Reference(self.Info)
         self.fontMapping = {}
@@ -208,12 +200,14 @@ class PDFDocument(PDFObject):
 
     def SaveToFile(self, filename, canvas):
         if getattr(self,'_savedToFile',False):
-            raise RunTimeError("class %s instances can only be saved once" % self.__class__.__name__)
+            raise RuntimeError("class %s instances can only be saved once" % self.__class__.__name__)
         self._savedToFile = True
         if hasattr(getattr(filename, "write",None),'__call__'):
             myfile = 0
             f = filename
-            filename = makeFileName(getattr(filename,'name',''))
+            filename = getattr(f,'name','')
+            if isinstance(filename,int): filename = '<os fd:%d>'% filename
+            filename = makeFileName(filename)
         else :
             myfile = 1
             filename = makeFileName(filename)
@@ -241,8 +235,9 @@ class PDFDocument(PDFObject):
         # prepare outline
         self.Reference(self.Catalog)
         self.Reference(self.info)
-        outline = self.outline
-        outline.prepare(self, canvas)
+        self.Outlines.prepare(self, canvas)
+        if self.Outlines.ready<0:
+            self.Catalog.Outlines = None
         return self.format()
 
     def inPage(self):
@@ -870,8 +865,8 @@ class PDFIndirectObject(PDFObject):
         document.encrypt.register(n, v)
         fcontent = format(self.content, document, toplevel=1)   # yes this is at top level
         return (pdfdocEnc("%s %s obj\n"%(n,v))
-                +fcontent+ (b'' if fcontent.endswith(b'\n') else b'\n')
-                +b'endobj\n')
+            +fcontent+ (b'' if fcontent.endswith(b'\n') else b'\n')
+            +b'endobj\n')
 
 class PDFObjectReference(PDFObject):
     def __init__(self, name):
@@ -1387,13 +1382,12 @@ class PDFOutlines(PDFObject):
         self.mydestinations = destinationtree
 
     def format(self, document):
+        #this should never be called if None in (self.first,self.last)
         D = {}
         D["Type"] = PDFName("Outlines")
-        c = self.count
-        D["Count"] = c
-        if c!=0:
-            D["First"] = self.first
-            D["Last"] = self.last
+        D["Count"] = self.count
+        D["First"] = self.first
+        D["Last"] = self.last
         PD = PDFDictionary(D)
         return PD.format(document)
 
@@ -1444,7 +1438,7 @@ class PDFOutlines(PDFObject):
             else:
                 self.first = self.last = None
                 self.count = 0
-                self.ready = 1
+                self.ready = -1
                 return
         #self.first = document.objectReference("Outline.First")
         #self.last = document.objectReference("Outline.Last")
@@ -1458,7 +1452,7 @@ class PDFOutlines(PDFObject):
             levelname = "Outline"
             Parent = document.Reference(document.Outlines)
         else:
-            self.count = self.count+1
+            self.count += 1
             levelname = "Outline.%s" % self.count
             if Parent is None:
                 raise ValueError("non-top level outline elt parent must be specified")
@@ -1538,7 +1532,6 @@ class PDFInfo(PDFObject):
     _dateFormatter = None
 
     def __init__(self):
-        self.invariant = rl_config.invariant
         self.trapped = 'False'  #could be 'True' or 'Unknown'
 
     def digest(self, md5object):
@@ -1550,7 +1543,7 @@ class PDFInfo(PDFObject):
         D = {}
         D["Title"] = PDFString(self.title)
         D["Author"] = PDFString(self.author)
-        D['ModDate'] = D["CreationDate"] = PDFDate(invariant=self.invariant,dateFormatter=self._dateFormatter)
+        D['ModDate'] = D["CreationDate"] = PDFDate(ts=document._timeStamp,dateFormatter=self._dateFormatter)
         D["Producer"] = PDFString(self.producer)
         D["Creator"] = PDFString(self.creator)
         D["Subject"] = PDFString(self.subject)
@@ -1728,27 +1721,19 @@ class PDFRectangle(PDFObject):
         A = PDFArray([self.llx, self.lly, self.ulx, self.ury])
         return format(A, document)
 
-_NOWT=None
-def _getTimeStamp():
-    global _NOWT
-    if not _NOWT:
-        _NOWT = time.time()
-    return _NOWT
-
 class PDFDate(PDFObject):
     # gmt offset now suppported properly
-    def __init__(self, invariant=rl_config.invariant, dateFormatter=None):
-        if invariant:
-            now = (2000,1,1,0,0,0,0)
-            self.dhh = 0
-            self.dmm = 0
-        else:
-            now = tuple(time.localtime(_getTimeStamp())[:6])
-            self.dhh = int(time.timezone / (3600.0))
-            self.dmm = (time.timezone % 3600) % 60
-        self.date = now[:6]
+    def __init__(self, ts=None, dateFormatter=None):
+        if ts is None:
+            ts = TimeStamp()
+        self._init(ts)
         self.dateFormatter = dateFormatter
 
+    def _init(self,ts):
+        self.date = ts.YMDhms
+        self.dhh = ts.dhh
+        self.dmm = ts.dmm
+
     def format(self, doc):
         dfmt = self.dateFormatter or (
                 lambda yyyy,mm,dd,hh,m,s:
